==> grammatica-0.3.tar.gz: grammatica-0.3/src/java/net/percederberg/grammatica/parser/re/StringElement.java <==
/*
 * StringElement.java
 *
 * This work is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 * As a special exception, the copyright holders of this library give
 * you permission to link this library with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the
 * library, but you are not obligated to do so. If you do not wish to
 * do so, delete this exception statement from your version.
 *
 * Copyright (c) 2003 Per Cederberg. All rights reserved.
 */

package net.percederberg.grammatica.parser.re;

import java.io.PrintWriter;

/**
 * A regular expression string element. This element only matches an
 * exact string. Once created, the string element is immutable.
 *
 * @author   Per Cederberg, <per at percederberg dot net>
 * @version  1.0
 */
class StringElement extends Element {

    /**
     * The string to match with.
     */
    private String value = null;
    
    /**
     * Creates a new string element.
     * 
     * @param c              the character to match with
     */
    public StringElement(char c) {
        this(String.valueOf(c));
    }

    /**
     * Creates a new string element.
     * 
     * @param str            the string to match with
     */
    public StringElement(String str) {
        value = str;
    }

    /**
     * Returns the string to be matched.
     * 
     * @return the string to be matched
     */
    public String getString() {
        return value;
    }

    /**
     * Returns this element as it is immutable. 
     * 
     * @return this string element
     */
    public Object clone() {
        return this;
    }

    /**
     * Returns the length of a matching string starting at the 
     * specified position. The number of matches to skip can also be
     * specified, but numbers higher than zero (0) cause a failed 
     * match for any element that doesn't attempt to combine other 
     * elements.
     *
     * @param m              the matcher being used 
     * @param str            the string to match
     * @param start          the starting position
     * @param skip           the number of matches to skip
     * 
     * @return the length of the longest matching string, or
     *         -1 if no match was found
     */
    public int match(Matcher m, CharBuffer str, int start, int skip) {
        if (skip != 0) {
            return -1;
        }
        for (int i = 0; i < value.length(); i++) {
            if (start + i >= str.length()) {
                m.setReadEndOfString();
                return -1;
            }
            if (str.charAt(start + i) != value.charAt(i)) {
                return -1;
            }
        }
        return value.length();
    }

    /**
     * Prints this element to the specified output stream.
     * 
     * @param out            the output stream to use
     * @param indent         the current indentation
     */
    public void printTo(PrintWriter out, String indent) {
        out.println(indent + "'" + value + "'");
    }

}

==> grammatica-1.3.tar.gz: grammatica-1.3/src/java/net/percederberg/grammatica/parser/re/StringElement.java <==
/*
 * StringElement.java
 *
 * This work is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 * As a special exception, the copyright holders of this library give
 * you permission to link this library with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the
 * library, but you are not obligated to do so. If you do not wish to
 * do so, delete this exception statement from your version.
 *
 * Copyright (c) 2003 Per Cederberg. All rights reserved.
 */

package net.percederberg.grammatica.parser.re;

import java.io.PrintWriter;

/**
 * A regular expression string element. This element only matches an
 * exact string. Once created, the string element is immutable.
 *
 * @author   Per Cederberg, <per at percederberg dot net>
 * @version  1.0
 */
class StringElement extends Element {

    /**
     * The string to match with.
     */
    private String value = null;
    
    /**
     * Creates a new string element.
     * 
     * @param c              the character to match with
     */
    public StringElement(char c) {
        this(String.valueOf(c));
    }

    /**
     * Creates a new string element.
     * 
     * @param str            the string to match with
     */
    public StringElement(String str) {
        value = str;
    }

    /**
     * Returns the string to be matched.
     * 
     * @return the string to be matched
     */
    public String getString() {
        return value;
    }

    /**
     * Returns this element as it is immutable. 
     * 
     * @return this string element
     */
    public Object clone() {
        return this;
    }

    /**
     * Returns the length of a matching string starting at the 
     * specified position. The number of matches to skip can also be
     * specified, but numbers higher than zero (0) cause a failed 
     * match for any element that doesn't attempt to combine other 
     * elements.
     *
     * @param m              the matcher being used 
     * @param str            the string to match
     * @param start          the starting position
     * @param skip           the number of matches to skip
     * 
     * @return the length of the longest matching string, or
     *         -1 if no match was found
     */
    public int match(Matcher m, CharBuffer str, int start, int skip) {
        if (skip != 0) {
            return -1;
        }
        for (int i = 0; i < value.length(); i++) {
            if (start + i >= str.length()) {
                m.setReadEndOfString();
                return -1;
            }
            if (str.charAt(start + i) != value.charAt(i)) {
                return -1;
            }
        }
        return value.length();
    }

    /**
     * Prints this element to the specified output stream.
     * 
     * @param output         the output stream to use
     * @param indent         the current indentation
     */
    public void printTo(PrintWriter output, String indent) {
        output.println(indent + "'" + value + "'");
    }

}
